# -*- coding: UTF-8 -*-
#
# Запрос и контроль выгрузки продаж
#


from plugins import status
from propertyReader import properties
import logger
import optparse
import queuesManager as queuesManager
import requests
import sendSalesModule as sendSalesModule
import sys
import traceback

loggerSync = logger.getLogger()


def unloadSalesInSyncMode():
    loggerSync.info("Выгрузка продаж будет произведена в синхронном режиме")
    unloadStatus, successUnloadedSalesIds, messEn, messRu = sendSalesModule.sendGrpcUnloadedSales()
    loggerSync.info(messRu)
    queuesManager.removeUnloadedSalesByGrpc(successUnloadedSalesIds)
    if unloadStatus not in [status.OK, status.empty]:
        # Если по какой-либо причине не удалось отправить сообщение на сервер - оно не отправится до тех пор, пока не сбросится ошибочное состояние.
        # Сбросить его можно при перезапуске sync-а или при получении нового запроса на выгрузку продаж через thrift
        loggerSync.info('Выгрузка продаж в синхронном режиме завершилась с ошибкой')
        print(messRu)
        tasks = queuesManager.updateUnloadedSalesTaskState(True)
        try:
            for task in tasks:
                if task.get('saleStateId'):
                    status.updateSaleState(task.get('saleStateId'), messEn, messRu, status.STATUS_ERROR,
                                           properties.httpHostCS, properties.httpPortCS)
        except Exception as e:
            loggerSync.warn("Exception while send status on server: %s", e.message)
            loggerSync.warn(traceback.format_exc())
        return False
    else:
        loggerSync.info('Выгрузка продаж в синхронном режиме завершилась успешно')
        return True


def sendUnloadRequestToCS(host, port, cashId):
    """ Отправка запроса на КС для выгрузки продаж """
    loggerSync.info('Отправка запроса на КС для выгрузки продаж')
    host = host if host else properties.httpHostCS
    port = port if port else properties.httpPortCS
    try:
        url = 'http://%s:%s/CSrest/rest/sales/unload/cashid/%s/kick' % (host, port, cashId)
        loggerSync.info('URL: %s, method: PUT' % url)
        with requests.Session() as session:
            response = session.put(url, headers={'Content-Type': 'application/json', 'Accept': 'application/json'}, timeout=60)
        if response.status_code == requests.codes.ok:
            loggerSync.info('Запрос успешно отправлен')
    except Exception as e:
        loggerSync.error('Ошибка при запросе выгрузки продаж через рест на КС: %s', str(e).encode('utf8'))


def sendSalesRequest(options):
    syncMode = options.syncMode and properties.protocolVersion >= 4

    if properties.unloadSalesRequestThroughCS and not syncMode:
        sendUnloadRequestToCS(properties.httpHostCS, properties.httpPortCS, properties.cashCode)
        return 0
    else:
        messEn = 'Start unload'
        messRu = 'Начинаем выгрузку продаж'
        saleStateId = None
        try:
            stateParams = {"callFromCS": False}
            unloadParams = {"unloadingShopId": options.shopCode, "unloadingCashCode": options.cashCode}
            if options.dates:
                stateParams["fromDate"] = options.dates.split('@')[0]
                stateParams["toDate"] = options.dates.split('@')[1]
                unloadParams["dates"] = options.dates
                unloadParams["closedShifts"] = options.closedShifts
            elif options.shifts:
                stateParams["fromShiftNum"] = options.shifts.split('@')[0]
                stateParams["toShiftNum"] = options.shifts.split('@')[1]
                unloadParams["shifts"] = options.shifts
                unloadParams["closedShifts"] = options.closedShifts
            else:
                loggerSync.warn("Время выгрузки продаж или диапазон смен не указаны. \nВызываем выгрузку продаж за последние 2 часа")
                stateParams["fromDate"] = sendSalesModule.CSReguestBegin
                stateParams["toDate"] = sendSalesModule.CSReguestEnd
                unloadParams["dates"] = '%s@%s' % (sendSalesModule.CSReguestBegin, sendSalesModule.CSReguestEnd)

            saleStateId = status.addSaleState(messEn, messRu, status.STATUS_DURING, properties.httpHostCS, properties.httpPortCS, **stateParams)
            unloadParams["saleStateId"] = saleStateId
            rez, messEn, messRu = sendSalesModule.main(**unloadParams)
            saleStatus = status.STATUS_ERROR
            if rez == status.OK:
                saleStatus = status.STATUS_READY
            elif rez == status.start:
                saleStatus = status.STATUS_DURING
            status.updateSaleState(saleStateId, messEn, messRu, saleStatus, properties.httpHostCS, properties.httpPortCS)
        except Exception as e:
            print('Ошибка обращения к серверу')
            status.updateSaleState(saleStateId, str(e), str(e), status.STATUS_ERROR, properties.httpHostCS, properties.httpPortCS)
            if syncMode:
                return 1

    return 1 if syncMode and not unloadSalesInSyncMode() else 0


if __name__ == '__main__':
    parser = optparse.OptionParser()
    parser.add_option("-s", "--shifts", dest="shifts", help="@-separated shift range")
    parser.add_option("-d", "--dates", dest="dates", help="@-separated dates range. Date format: YY-mm-ddTHH:MM:SS.0")
    parser.add_option("-c", "--closedShifts", dest="closedShifts", default=0, type='int',
                      help="Unload only closed, opened or closed and opened shifts. Use: 0-closed, 1-opened, 2-closedAndOpened  [default: %default]")
    parser.add_option("-K", "--cashCode", dest="cashCode", help="Unload for this cash (Parameter only for server mode)")
    parser.add_option("-S", "--shopCode", dest="shopCode", help="Unload for this shop (Parameter only for server mode)")
    parser.add_option("-n", "--sync", action="store_true", dest="syncMode", default = False, help="Unload sales in synchronized mode")
    (options, args) = parser.parse_args(sys.argv[1:])
    resultCode = sendSalesRequest(options)
    sys.exit(resultCode)
